package com.persagy.apm.energyalarmstarter.alarmengine.jms;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.persagy.apm.energyalarmstarter.alarmdata.mq.model.DmpMessage;
import com.persagy.apm.energyalarmstarter.alarmengine.constants.RedisConstants;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.AlarmCondition;
import com.persagy.apm.energyalarmstarter.alarmengine.feign.ObjConditionRel;
import com.persagy.apm.energyalarmstarter.alarmengine.jms.model.ItemCodeCondition;
import com.persagy.apm.energyalarmstarter.alarmengine.jms.model.ObjConditionInfo;
import com.persagy.apm.energyalarmstarter.alarmengine.redis.RedisUtil;
import com.persagy.apm.energyalarmstarter.alarmengine.service.AlarmRecordMsgHandler;
import io.netty.channel.ChannelHandler;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;

import javax.annotation.Resource;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Map;

/**
 * @description: 报警引擎消息处理类
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/11/30 10:31 上午
 * @version: V1.0
 */
@Slf4j
@ChannelHandler.Sharable
public class AlarmEngineMsgHandler {
    @Autowired
    private AlarmRecordMsgHandler alarmRecordMsgHandler;

    @Resource(name = "alarmRedisTemplate")
    private RedisTemplate<String, String> alarmRedisTemplate;

    @Autowired
    private RedisUtil redisUtil;

    @SneakyThrows
    public String getAlarmCategory(DmpMessage dmpMessage) {
        JSONObject data = dmpMessage.getExts();
        String itemCode = data.getString("itemCode");
        return alarmRecordMsgHandler.getAlarmCategory(alarmRecordMsgHandler.getAlarmUrlParam(dmpMessage), itemCode);
    }

    /**
     * @description: 创建报警记录，并发送报警记录id至边缘端
     * @param: nettyMessage
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 3:33 下午
     * @version: V1.0
     */
    @SneakyThrows
    public void createAlarmRecordAndSendRecordId(DmpMessage dmpMessage) {
        JSONObject data = dmpMessage.getExts();
        data.put("userId", "system");
        alarmRecordMsgHandler.createAlarm(data);
    }

    /**
     * @description: 更新报警记录
     * @param: message
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/11/30 4:13 下午
     * @version: V1.0
     */
    @SneakyThrows
    public void updateAlarmRecord(DmpMessage dmpMessage) {
        JSONObject data = dmpMessage.getExts();
        data.put("userId", "system");
        alarmRecordMsgHandler.updateAlarmRecord(data);
    }

    /**
     * @description: 报警仍在持续的处理方法
     * @param: nettyMessage
     * @return: void
     * @exception:
     * @author: lixing
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/12/17 4:44 下午
     * @version: V1.0
     */
    public void alarmContinue(DmpMessage dmpMessage) {
        log.info("报警仍在继续：{}", dmpMessage.toString());
    }

    /**
     * 同步新的报警条件
     *
     * @param msg 报警条件消息
     * @author lixing
     * @version V1.0 2021/10/26 7:51 下午
     */
    public void syncNewCondition(DmpMessage msg) {
        AlarmCondition alarmCondition = JSONObject.parseObject(msg.getStr1(), AlarmCondition.class);
        // TODO: 2021/11/16  更新redis
        alarmRedisTemplate.opsForHash().put("ALARM_CONDITION", alarmCondition.getId(), JSON.toJSONString(alarmCondition));
    }

    /**
     * 同步更新的报警条件
     *
     * @param msg 报警条件消息
     * @author lixing
     * @version V1.0 2021/10/26 7:51 下午
     */
    public void syncUpdatedCondition(DmpMessage msg) {
        AlarmCondition alarmCondition = JSONObject.parseObject(msg.getStr1(), AlarmCondition.class);
        // TODO: 2021/11/16  更新redis
        alarmRedisTemplate.opsForHash().delete("ALARM_CONDITION",alarmCondition.getId());
        alarmRedisTemplate.opsForHash().put("ALARM_CONDITION", alarmCondition.getId(), JSON.toJSONString(alarmCondition));
    }

    /**
     * 同步删除的报警条件
     *
     * @param msg 报警条件消息
     * @author lixing
     * @version V1.0 2021/10/26 7:51 下午
     */
    public void syncDeletedCondition(DmpMessage msg) {
        // TODO: 2021/11/16 更新redis
        alarmRedisTemplate.opsForHash().delete("ALARM_CONDITION",msg.getStr1());

    }

    /**
     * 同步新的设备与报警条件关联关系
     *
     * @param msg 关联关系消息
     * @author lixing
     * @version V1.0 2021/10/26 7:51 下午
     */
    public void syncNewObjConditionRelList(DmpMessage msg) {
        List<ObjConditionRel> createdRelList = JSONObject.parseArray(msg.getStr1(), ObjConditionRel.class);
        syncCreatedObjConditionRelList2Redis(createdRelList);
    }

    /**
     * 新的设备与报警条件关联关系同步至redis
     *
     * @param createdRelList 新增的关联关系
     * @author lixing
     * @version V1.0 2021/11/20 5:29 下午
     */
    private void syncCreatedObjConditionRelList2Redis(List<ObjConditionRel> createdRelList) {
        if (CollectionUtils.isEmpty(createdRelList)) {
            return;
        }
        for (ObjConditionRel objConditionRel : createdRelList) {
            // 报警对象id
            String objId = objConditionRel.getObjId();
            // 报警对象和报警条件的关联信息
            ObjConditionInfo objConditionInfo = redisUtil.get(RedisConstants.OBJ_CONDITION_REL, objId, ObjConditionInfo.class);
            if (objConditionInfo == null) {
                objConditionInfo = new ObjConditionInfo(objConditionRel.getProjectId());
            }
            ItemCodeCondition condition = new ItemCodeCondition();
            condition.setConditionId(objConditionRel.getConditionId());
            condition.setItemCode(objConditionRel.getItemCode());
            // 对象内部自己做去重操作
            objConditionInfo.addCondition(condition);
            redisUtil.put(RedisConstants.OBJ_CONDITION_REL, objId, JSON.toJSONString(objConditionInfo));
        }
    }

    /**
     * 删除的设备与报警条件关联关系同步至redis
     *
     * @param deletedRelList 删除的关联关系
     * @author lixing
     * @version V1.0 2021/11/20 5:29 下午
     */
    private void syncDeletedObjConditionRelList2Redis(List<ObjConditionRel> deletedRelList) {
        if (CollectionUtils.isEmpty(deletedRelList)) {
            return;
        }
        for (ObjConditionRel objConditionRel : deletedRelList) {
            // 报警对象id
            String objId = objConditionRel.getObjId();
            ObjConditionInfo objConditionInfo = redisUtil.get(RedisConstants.OBJ_CONDITION_REL, objId, ObjConditionInfo.class);
            ItemCodeCondition condition = new ItemCodeCondition();
            condition.setConditionId(objConditionRel.getConditionId());
            condition.setItemCode(objConditionRel.getItemCode());
            objConditionInfo.removeCondition(condition);
            if (CollectionUtils.isEmpty(objConditionInfo.getConditions())) {
                redisUtil.delete(RedisConstants.OBJ_CONDITION_REL, objId);
            } else {
                redisUtil.put(RedisConstants.OBJ_CONDITION_REL, objId, JSON.toJSONString(objConditionInfo));
            }
        }
    }

    /**
     * 同步删除的设备与报警条件关联关系
     *
     * @param msg 关联关系消息
     * @author lixing
     * @version V1.0 2021/10/26 7:51 下午
     */
    public void syncDeletedObjConditionRelList(DmpMessage msg) {
        List<ObjConditionRel> deletedRelList = JSONObject.parseArray(msg.getStr1(), ObjConditionRel.class);
        syncDeletedObjConditionRelList2Redis(deletedRelList);
    }

    public void syncUpdateObjConditionRelList(DmpMessage msg) {
        List<ObjConditionRel> deletedRelList = JSONObject.parseArray(msg.getStr1(), ObjConditionRel.class);
        List<ObjConditionRel> createdRelList = JSONObject.parseArray(msg.getStr2(), ObjConditionRel.class);
        // 先删除，再创建
        syncDeletedObjConditionRelList2Redis(deletedRelList);
        syncCreatedObjConditionRelList2Redis(createdRelList);
    }

    /**
     * 增量同步报警定义
     *
     * @author lixing
     * @version V1.0 2021/11/20 3:08 下午
     */
    @SneakyThrows
    public void incrementSyncAlarmConfig(DmpMessage dmpMessage) {
        Map<String, JSONArray> changedAlarmConfigs = alarmRecordMsgHandler.queryChangedAlarmConfigs(dmpMessage);
        JSONArray createdConfigUniques = changedAlarmConfigs.get("createdConfigUniques");
        JSONArray deletedConfigUniques = changedAlarmConfigs.get("deletedConfigUniques");
        if (!CollectionUtils.isEmpty(deletedConfigUniques)) {
            // 处理删除的报警定义 // TODO: 2021/11/20
            // 报警定义中不包含报警条件的id
            // 需要根据报警定义中的报警类型，从设备与报警条件的关联关系中找到对应的报警条件id。
            // 删除报警条件
            // 删除关联关系
        }
        if (!CollectionUtils.isEmpty(createdConfigUniques)) {
            // 处理新增的报警定义 // TODO: 2021/11/20
        }
    }
}
